# coding: utf-8
require 'spec_helper'

describe Libmagellan::MQTT do

  let(:consumer_key) { "groovenauts.app1" }
  let(:token){ "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" }
  let(:lm) {
    lm = Libmagellan::MQTT.new({host: "127.0.0.1",
                                port: 1883,
                                consumer_key: consumer_key,
                                consumer_secret: "test",
                                client_version: "0.0.1"
                               })
    lm.token = token
    lm
  }
  let(:socket) do
    socket = StringIO.new
    if socket.respond_to?(:set_encoding)
      socket.set_encoding("binary")
    else
      socket
    end
  end
  let(:client){ MQTT::Client.new(:host => 'localhost') }



  describe :publish do
    before do
      allow(TCPSocket).to receive(:new).and_return(socket)
      allow(Thread).to receive(:new)
      client.instance_variable_set('@socket', socket)
      lm.instance_variable_set('@client', client)
    end

    it "is success" do
      lm.publish("topic", "payload")
      expect(socket.string).to eq("\x30\x0e\x00\x05topicpayload".b)
    end
  end


  describe :subscribe do
    before do
      client.instance_variable_set('@socket', socket)
      lm.instance_variable_set('@client', client)
    end

    it "is success" do
      lm.subscribe('a/b')
      expect(socket.string).to eq("\x82\x08\x00\x01\x00\x03a/b\x00".b)
    end

    it "is success" do
      lm.subscribe(['a/b'])
      expect(socket.string).to eq("\x82\x08\x00\x01\x00\x03a/b\x00".b)
    end
  end


  describe :get do
    before do
      client.instance_variable_set('@socket', socket)
      lm.instance_variable_set('@client', client)
    end

    it "is success" do
      inject_packet(:topic => 'topic0', :payload => 'payload0', :qos => 0)
      topic,payload = lm.get
      expect(topic).to eq('topic0')
      expect(payload).to eq('payload0')
    end
  end


  private

  def inject_packet(opts={})
    packet = MQTT::Packet::Publish.new(opts)
    client.instance_variable_get('@read_queue').push(packet)
  end

end
